/* This file is part of the Neper software package. */
/* Copyright (C) 2003-2014, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"net_germ_.h"

void
net_germ (struct IN In, int level, struct TESS Tess, int poly, struct GERMSET *pGSet)
{
  int status;
  struct TESS Cell;

  neut_tess_set_zero (&Cell);
  if (Tess.Dim == 3)
    neut_tess_poly_tess (Tess, poly, &Cell);
  else if (Tess.Dim == 1)
    neut_tess_tess (Tess, &Cell);

  // Initializing variables --------------------------------------------

  neut_germset_free (pGSet);

  // Testing out input data --------------------------------------------

  // Regular distributions are only available for a cubic domain
  if ((!strcmp (In.morpho[Tess.Level + 1], "tocta")
       || !strcmp (In.morpho[Tess.Level + 1], "cube")
       || !strcmp (In.morpho[Tess.Level + 1], "dodeca"))
      && strncmp (In.domain, "cube", 4) != 0)
    ut_print_message
      (2, 0,
       "This morphology (%s) cannot be used with a non-cube domain (%s)\n",
       In.morpho[Tess.Level + 1], In.domain);

  if ((!strcmp (In.morpho[Tess.Level + 1], "square"))
      && (strncmp (In.domain, "cube", 4) != 0
       && strncmp (In.domain, "square", 6) != 0))
    ut_print_message
      (2, 0,
       "This morphology (%s) cannot be used with a non-cube domain (%s)\n",
       In.morpho[Tess.Level + 1], In.domain);

  // Setting up properties of the germset (number of germs, etc.) ------

  net_germ_prop (In, Tess, poly, pGSet);

  // Distributing germs ------------------------------------------------

  if (!strcmp ((*pGSet).morpho, "lamella"))
    net_germ_seed (Tess.Level, poly, In.n[Tess.Level + 1] + 2, pGSet);
  else
    net_germ_seed (Tess.Level, poly, NULL, pGSet);

  // weights
  if (strcmp ((*pGSet).weight, "none") != 0)
    ut_print_message (0, 3, "Computing weights...\n");

  if (!strcmp ((*pGSet).weight, "none"))
    (*pGSet).GermWeight = ut_alloc_1d ((*pGSet).N + 1);
  else if ((*pGSet).weight[0] == '@')
    net_germ_weight_custom ((*pGSet).weight + 1, pGSet);
  else
    net_germ_weight_rand (pGSet);

  if (strcmp ((*pGSet).weight, "none") != 0)
    ut_print_message (0, 3, "Computing positions...\n");

  // positions
  if (!strcmp ((*pGSet).morpho, "tocta") || !strcmp ((*pGSet).morpho, "cube")
   || !strcmp ((*pGSet).morpho, "square"))
    net_germ_reg (Cell, pGSet);
  else if ((*pGSet).morpho[0] == '@')
    net_germ_custom ((*pGSet).morpho + 1, In.dim, In.domain, Cell, pGSet);
  else if (!strcmp ((*pGSet).morpho, "lamella"))
    net_germ_lam (In.n[Tess.Level + 1] + 2, Cell, pGSet);
  else
  {
    if (!strcmp ((*pGSet).weight, "none"))
      net_germ_rand (In, level, Cell, pGSet);
    else
      net_germ_lag (In, Cell, pGSet);
  }

  if (!(*pGSet).GermWeight)
    (*pGSet).GermWeight = ut_alloc_1d ((*pGSet).N + 1);

  // Distributing germs outside the domain, if needed
  if (!strcmp (In.ttype, "periodic") || !strcmp (In.ttype, "subdomain"))
    net_germ_neigh (In, pGSet);

  // Randomize germ distribution ---------------------------------------

  if (In.randomizestring)
    net_germ_randomize (In, pGSet, Cell);

  // Computing orientations --------------------------------------------

  net_germ_ori (In, pGSet);

  // In case of weighted germs, checking that tessellation generation
  // will be possible
  if (strcmp ((*pGSet).weight, "none") != 0)
  {
    status = net_germ_weight_check (pGSet);

    if (status == -1)
      ut_print_message (2, 0,
			"Weighted germ distribution not valid (a germ lies within the sphere of another germ.");
  }

  // Free'ing memory ---------------------------------------------------

  neut_tess_free (&Cell);

  return;
}
