/* This file is part of the Neper software package. */
/* Copyright (C) 2003-2014, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"net_germ_custom_.h"

void
net_germ_custom_read (char *filename, int dim,
		      struct TESS Dom, struct GERMSET *pGSet)
{
  int i;
  double coo;
  FILE *file = NULL;

  (*pGSet).Size = ut_alloc_2d (3, 2);
  neut_tess_bbox (Dom, (*pGSet).Size);

  (*pGSet).GermCoo = ut_alloc_2d ((*pGSet).N + 1, 3);

  file = ut_file_open (filename, "R");
  ut_array_2d_fscanf (file, (*pGSet).GermCoo + 1, (*pGSet).N, dim);
  ut_file_close (file, filename, "R");

  if (dim == 2)
  {
    coo = .5 * ((*pGSet).Size[2][0] + (*pGSet).Size[2][1]);
    for (i = 1; i <= (*pGSet).N; i++)
      (*pGSet).GermCoo[i][2] = coo;
  }

  return;
}

void
net_germ_custom_fix_cylinder (char* domain,
			      struct TESS Dom, struct GERMSET *pGSet)
{
  int i, j, iter;
  double h, d, reps = 1e-6;
  char* tmp = ut_alloc_1d_char (strlen (domain) + 1);
  double* O = ut_alloc_1d (3);
  double* V = ut_alloc_1d (3);

  strcpy (tmp, domain);
  ut_string_fnr (tmp, '(', ' ');
  ut_string_fnr (tmp, ')', ' ');
  ut_string_fnr (tmp, ',', ' ');
  sscanf (tmp, "%*s%lf%lf", &h, &d);

  ut_array_1d_set_2 (O, .5 * d, .5 * d);

  for (i = 1; i <= (*pGSet).N; i++)
  {
    ut_array_1d_sub (O, (*pGSet).GermCoo[i], 2, V);

    if (ut_space_dist2d (O, (*pGSet).GermCoo[i]) < .5 * d)
    {
      iter = 0;
      while (!neut_tess_point_inpoly (Dom, (*pGSet).GermCoo[i], 1))
      {
	for (j = 0; j < 2; j++)
	  (*pGSet).GermCoo[i][j] -= reps * V[j];
	iter++;
      }
      if (iter > 0)
	ut_print_message (1, 2, "Germ %d moved by %g toward the domain axis.\n",
			  i, reps * .5 * d * iter);
    }
  }

  ut_free_1d_char (tmp);
  ut_free_1d (O);
  ut_free_1d (V);

  return;
}


void
net_germ_custom_test (struct TESS Dom, struct GERMSET* pGSet)
{
  int i;

  for (i = 1; i <= (*pGSet).N; i++)
    if (!neut_tess_point_incell (Dom, (*pGSet).GermCoo[i], 1))
      ut_print_message (2, 2, "Germ %d is not located within the domain.\n", i);

  return;
}
