/* This file is part of the Neper software package. */
/* Copyright (C) 2003-2014, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"net_polycomp_.h"
#include<ANN/ANN.h>

extern void PolyhedronCalculation (struct POLY Domain, struct GERMSET GermSet,
				   ANNkd_tree* kdTree, int PolyId,
				   struct POLY *pPoly, int* neighqty);

/* net_polycomp builds the set of Voronoi polyhedra from the set
 * of germ and the initial domain.
 */
void
net_polycomp (struct POLY Domain, struct GERMSET GermSet, struct POLY **pPoly,
	      int verbosity)
{
  int i, neighqty;
  double meanneighqty;
  char *message = (char*) calloc (10, sizeof (char));
  ANNkd_tree *kdTree;

  kdTree = new ANNkd_tree (GermSet.GermCoo + 1, GermSet.N,
      GermSet.Dim);

  // If *pPoly is not empty, free'ing
  neut_poly_array_free (pPoly, GermSet.N);

  // Alloc'ing Poly
  (*pPoly) = (POLY *) calloc (GermSet.N + 1, sizeof (POLY));

  // Calculating polys one by one
  meanneighqty = -1;
  neighqty = -1;
  for (i = 1; i <= GermSet.N; i++)
  {
    neighqty = ut_num_d2ri (meanneighqty);

    PolyhedronCalculation (Domain, GermSet, kdTree, i, &((*pPoly)[i]),
			   &neighqty);

    meanneighqty = (meanneighqty * (i - 1) + neighqty) / i;

    if (verbosity)
      ut_print_progress_nonl (stdout, i, GermSet.N, "%3.0f%%", message);
  }

  delete kdTree;
  annClose ();

  free (message);

  return;
}
